#!/bin/bash
#
# Simple script to add a GSS domain to Apache configuration
#
# Usage: $0 GSS_name GSS_domain_number domain_hostname
# kovacs.zoltan@mimoza.hu 2005-06-15

APACHEADMIN="webadmin"                                  # Linux user for web administration
APACHEADMINGROUP="webadmin"                             # Linux group for web administration
APACHEAUTHDIR="/etc/apache-ssl/auth.d"                  # Web user's authentication data folder
APACHECONFIGDIR="/etc/apache/conf.d"                    # Directory for apache virtualhosts
APACHEDAEMON="/etc/init.d/apache"                       # Start-stop script for apache
APACHEDOCROOT="/var/www"                                # Public web documentroot
APACHEGROUP="www-data"                                  # Linux group for web content
APACHELOGDIR="/var/log/apache"                          # Root directory for Apache logs
APACHESSLCONFIGDIR="/etc/apache-ssl/conf.d"             # Directory for apache-ssl virtualhosts
APACHESSLDAEMON="/etc/init.d/apache-ssl"                # Start-stop script for apache-ssl
APACHESSLIP="*"                                         # IP for virtualhosts administration
APACHESSLPORT=":443"                                    # SSL service port
APACHESYSTEMCONF="SYSTEM.conf"				# Apache main virtualhost configuration
APACHEUPLOADDIR="upload"                                # Special directory for uploader
APACHEUSER="www-data"                                   # Linux user for public webservice
APACHEWEBIP="*"                                         # IP for public web virtualhosts
APACHEWEBPORT=""                                        # HTTP service port
AWK="/usr/bin/awk"					# awk command call
AWSTATSLINK="awstats"                                   # Link name points to web statistics
AWSTATSWEBROOT="$APACHEDOCROOT/awstats"                 # Web statistics html document root
CAT="/bin/cat"						# cat command call
CHOWN="/bin/chown"					# chown command call
DEFAULT_AUDITOR="auditor"                               # Default auditor user (stats)
DEFAULT_EMAIL="root@localhost.localdomain"              # Default email address
DEFAULT_PAGE="index.html"                               # Default webpage
DEFAULT_TOMCAT="system"					# Default Tomcat instance
DEFAULT_WEBMASTER="webmaster"                           # Default webmaster user (DAV)
ENVSUBST="/usr/bin/envsubst"                            # envsubst command call
EXIT_ERR="1"                                            # Exit code on error
EXIT_SUCC="0"                                           # Exit code on success
GREP="/bin/grep"                                        # grep command call
GSSTEMPLATEDIR="/usr/share/website-manager/templates"	# Template directory
GSSHTTPTEMPLATE="http.gss.conf"				# Apache webserver config template
GSSHTTPSTEMPLATE="https.gss.conf"			# Apache-SSL webserver config template
LN="/bin/ln"						# ln command call
MASKUSER="\([0-9a-zA-Z._-]\)\+"                         # SED regexp: valid username
MASKWEB="\(\([0-9a-zA-Z_-]\)\+\.\)\+\([0-9a-zA-Z]\)\+"  # SED regexp: valid website name
MASKEMAIL="$MASKUSER@$MASKWEB"                          # SED regexp: valid e-mail address
SED="/bin/sed"                                          # sed command call
WSM="/usr/local/sbin/website-manager.sh"		# wsm script call

# Messages

MSG_CANNOTCREATEUSERS="Can't create Apache users!\n"
MSG_CREATED="Created file or directory: "
MSG_MISSING="Missing file or directory: "
MSG_MISSING_SSLIP="No IP for apache-ssl service!\n"
MSG_MISSING_SYSTEMHOST="Apache system host isn't found!\n"
MSG_MISSING_WEBIP="No IP for apache service!\n"
MSG_USAGE="Usage:   $0 GSS_name GSS_domain_number domain_hostname\nExample: $0 gss 2 www.example.com\n\n"

# Check templates

if [ ! -f $GSSTEMPLATEDIR/$GSSHTTPTEMPLATE ];  then echo -ne "$MSG_MISSING$GSSTEMPLATEDIR/$GSSHTTPTEMPLATE\n";  exit $EXIT_ERR; fi
if [ ! -f $GSSTEMPLATEDIR/$GSSHTTPSTEMPLATE ]; then echo -ne "$MSG_MISSING$GSSTEMPLATEDIR/$GSSHTTPSTEMPLATE\n"; exit $EXIT_ERR; fi

# Get command-line parameters

GSSNAME=$1; shift
if [ -z "$GSSNAME" ]; then echo -ne "$MSG_USAGE"; exit $EXIT_ERR >&2; fi

GSSNUMBER=$1; shift
if [ -z "$GSSNUMBER" ]; then echo -ne "$MSG_USAGE"; exit $EXIT_ERR >&2; fi

DOMAINNAME=$1; shift
if [ -z "$DOMAINNAME" ]; then echo -ne "$MSG_USAGE"; exit $EXIT_ERR >&2; fi

# Check corresponding Apache configuration

if [ ! -f $APACHECONFIGDIR/$GSSNAME ]; then 
    echo -ne "$MSG_CREATED$APACHECONFIGDIR/$GSSNAME\n"
    touch $APACHECONFIGDIR/$GSSNAME
fi
if [ ! -f $APACHESSLCONFIGDIR/$GSSNAME ]; then 
    echo -ne "$MSG_CREATED$APACHESSLCONFIGDIR/$GSSNAME\n"
    touch $APACHESSLCONFIGDIR/$GSSNAME
fi

# Get parameters

if [ ! -f $APACHECONFIGDIR/$APACHESYSTEMCONF ]; then echo -ne "$MSG_MISSING$APACHECONFIGDIR/$APACHESYSTEMCONF\n"; exit $EXIT_ERR; fi
parameter=`$CAT $APACHECONFIGDIR/$APACHESYSTEMCONF | $GREP -i -m 1 "servername" | \
           $AWK '{ print $2 }'`
if [ -z "$parameter" ]; then echo -ne "$MSG_MISSING_SYSTEMHOST"; exit $EXIT_ERR >&2; fi
systemhost=$parameter

parameter=`$CAT $APACHECONFIGDIR/$GSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\([[:space:]]\+\)\(\(\([[:digit:]]\{1,3\}\)\.\)\{3\}[[:digit:]]\{1,3\}\)[^0-9].*$/\2/p"`
APACHEWEBIP=${parameter:-$APACHEWEBIP}
if [ -z "$APACHEWEBIP" ];  then echo -ne "$MSG_MISSING_WEBIP"; exit $EXIT_ERR >&2; fi

parameter=`$CAT $APACHECONFIGDIR/$GSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\(\:[[:digit:]]\{1,5\}\)[^0-9].*$/\1/p"`
APACHEWEBPORT=${parameter:-$APACHEWEBPORT}

parameter=`$CAT $APACHESSLCONFIGDIR/$GSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\([[:space:]]\+\)\(\(\([[:digit:]]\{1,3\}\)\.\)\{3\}[[:digit:]]\{1,3\}\)[^0-9].*$/\2/p"`
APACHESSLIP=${parameter:-$APACHESSLIP}
if [ -z "$APACHESSLIP" ];  then echo -ne "$MSG_MISSING_SSLIP"; exit $EXIT_ERR >&2; fi

parameter=`$CAT $APACHESSLCONFIGDIR/$GSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\(\:[[:digit:]]\{1,5\}\)[^0-9].*$/\1/p"`
APACHESSLPORT=${parameter:-$APACHESSLPORT}

parameter=`$CAT $APACHECONFIGDIR/$GSSNAME | $GREP -i -m 1 "serveradmin" | \
           $AWK '{ print $2 }'`
email=${parameter:-$DEFAULT_EMAIL}

parameter=`$CAT $APACHECONFIGDIR/$GSSNAME | $GREP -i -m 1 "jkmount" | \
           $AWK '{ print $3 }'`
tomcatname=${parameter:-$DEFAULT_TOMCAT}

vhost=$DOMAINNAME

# Apache webserver configuration

DATE=`date '+%Y-%m-%d %H:%M'`
export APACHEDOCROOT APACHELOGDIR APACHEUPLOADDIR APACHEWEBIP APACHEWEBPORT \
       DATE GSSNAME GSSNUMBER email systemhost tomcatname vhost; \
$CAT $GSSTEMPLATEDIR/$GSSHTTPTEMPLATE | $ENVSUBST >>$APACHECONFIGDIR/$GSSNAME 
$APACHEDAEMON reload

# Apache SSL webserver configuration

DATE=`date '+%Y-%m-%d %H:%M'`
export APACHEAUTHDIR APACHEDOCROOT APACHELOGDIR APACHEUPLOADDIR APACHESSLIP APACHESSLPORT \
       DATE GSSNAME GSSNUMBER email systemhost vhost; \
$CAT $GSSTEMPLATEDIR/$GSSHTTPSTEMPLATE | $ENVSUBST >>$APACHESSLCONFIGDIR/$GSSNAME 
$APACHESSLDAEMON reload

# Creating Apache users and groups

if [ -x $WSM ]; then
    $WSM -au  $GSSNAME d$GSSNUMBER-auditor >&2
    $WSM -ag  $GSSNAME d$GSSNUMBER-stats
    $WSM -aug $GSSNAME d$GSSNUMBER-auditor d$GSSNUMBER-stats
    $WSM -au  $GSSNAME d$GSSNUMBER-webmaster >&2
    $WSM -ag  $GSSNAME d$GSSNUMBER-admin
    $WSM -aug $GSSNAME d$GSSNUMBER-webmaster d$GSSNUMBER-admin
else
    echo -ne "$MSG_MISSING $WSM\n" >&2
    echo -ne "$MSG_CANNOTCREATEUSERS" >&2
fi

# Creating AwStat's link

if [ ! -L $APACHEDOCROOT/$GSSNAME/domain$GSSNUMBER/$AWSTATSLINK ]; then
    $LN -s $AWSTATSWEBROOT/$vhost $APACHEDOCROOT/$GSSNAME/domain$GSSNUMBER/$AWSTATSLINK
    $CHOWN $APACHEADMIN:$APACHEADMINGROUP $APACHEDOCROOT/$GSSNAME/domain$GSSNUMBER/$AWSTATSLINK
    echo -ne "$MSG_CREATED$APACHEDOCROOT/$GSSNAME/domain$GSSNUMBER/$AWSTATSLINK \n"
fi

# That's all, folks! :-)
